@@ -71,6 +71,10 @@ OperationParameters* AlgorithmCiftiCorrelation::getParameters()
     
     ret->createOptionalParameter(5, "-fisher-z", "apply fisher small z transform (ie, artanh) to correlation");
     
+    ret->createOptionalParameter(7, "-no-demean", "instead of correlation, do dot product of rows, then normalize by diagonal");
+    
+    ret->createOptionalParameter(8, "-covariance", "compute covariance instead of correlation");
+    
     OptionalParameter* memLimitOpt = ret->createOptionalParameter(6, "-mem-limit", "restrict memory usage");
     memLimitOpt->addDoubleParameter(1, "limit-GB", "memory limit in gigabytes");
     
@@ -167,24 +171,30 @@ void AlgorithmCiftiCorrelation::useParameters(OperationParameters* myParams, Pro
             throw AlgorithmException("memory limit cannot be negative");
         }
     }
+    bool noDemean = myParams->getOptionalParameter(7)->m_present;
+    bool covariance = myParams->getOptionalParameter(8)->m_present;
     if (roiOverrideMode)
     {
         if (ciftiRoiMode)
         {
-            AlgorithmCiftiCorrelation(myProgObj, myCifti, myCiftiOut, ciftiRoi, weights, fisherZ, memLimitGB);
+            AlgorithmCiftiCorrelation(myProgObj, myCifti, myCiftiOut, ciftiRoi, weights, fisherZ, memLimitGB, noDemean);
         } else {
-            AlgorithmCiftiCorrelation(myProgObj, myCifti, myCiftiOut, leftRoi, rightRoi, cerebRoi, volRoi, weights, fisherZ, memLimitGB);
+            AlgorithmCiftiCorrelation(myProgObj, myCifti, myCiftiOut, leftRoi, rightRoi, cerebRoi, volRoi, weights, fisherZ, memLimitGB, noDemean);
         }
     } else {
-        AlgorithmCiftiCorrelation(myProgObj, myCifti, myCiftiOut, weights, fisherZ, memLimitGB);
+        AlgorithmCiftiCorrelation(myProgObj, myCifti, myCiftiOut, weights, fisherZ, memLimitGB, noDemean, covariance);
     }
 }
 
 AlgorithmCiftiCorrelation::AlgorithmCiftiCorrelation(ProgressObject* myProgObj, const CiftiFile* myCifti, CiftiFile* myCiftiOut, const vector<float>* weights,
-                                                     const bool& fisherZ, const float& memLimitGB) : AbstractAlgorithm(myProgObj)
+                                                     const bool& fisherZ, const float& memLimitGB, const bool& noDemean, const bool& covariance) : AbstractAlgorithm(myProgObj)
 {
     LevelProgress myProgress(myProgObj);
-    init(myCifti, weights);
+    if (covariance)
+    {
+        if (fisherZ) throw AlgorithmException("cannot apply fisher z transformation to covariance");
+    }
+    init(myCifti, weights, noDemean, covariance);
     int numRows = myCifti->getNumberOfRows();
     CiftiXMLOld newXML = myCifti->getCiftiXMLOld();
     newXML.applyColumnMapToRows();
@@ -276,10 +286,15 @@ AlgorithmCiftiCorrelation::AlgorithmCiftiCorrelation(ProgressObject* myProgObj,
 
 AlgorithmCiftiCorrelation::AlgorithmCiftiCorrelation(ProgressObject* myProgObj, const CiftiFile* myCifti, CiftiFile* myCiftiOut,
                                                      const MetricFile* leftRoi, const MetricFile* rightRoi, const MetricFile* cerebRoi,
-                                                     const VolumeFile* volRoi, const vector<float>* weights, const bool& fisherZ, const float& memLimitGB) : AbstractAlgorithm(myProgObj)
+                                                     const VolumeFile* volRoi, const vector<float>* weights, const bool& fisherZ, const float& memLimitGB,
+                                                     const bool& noDemean, const bool& covariance) : AbstractAlgorithm(myProgObj)
 {
     LevelProgress myProgress(myProgObj);
-    init(myCifti, weights);
+    if (covariance)
+    {
+        if (fisherZ) throw AlgorithmException("cannot apply fisher z transformation to covariance");
+    }
+    init(myCifti, weights, noDemean, covariance);
     const CiftiXMLOld& origXML = myCifti->getCiftiXMLOld();
     if (origXML.getColumnMappingType() != CIFTI_INDEX_TYPE_BRAIN_MODELS)
     {
@@ -459,7 +474,8 @@ AlgorithmCiftiCorrelation::AlgorithmCiftiCorrelation(ProgressObject* myProgObj,
 }
 
 AlgorithmCiftiCorrelation::AlgorithmCiftiCorrelation(ProgressObject* myProgObj, const CiftiFile* myCifti, CiftiFile* myCiftiOut, const CiftiFile* ciftiRoi,
-                                                     const vector<float>* weights, const bool& fisherZ, const float& memLimitGB): AbstractAlgorithm(NULL)//HACK: get around the sentinel by passing a null, because this implementation calls another
+                                                     const vector<float>* weights, const bool& fisherZ, const float& memLimitGB,
+                                                     const bool& noDemean, const bool& covariance): AbstractAlgorithm(NULL)//HACK: get around the sentinel by passing a null, because this implementation calls another
 {
     const CiftiXML& roiXML = ciftiRoi->getCiftiXML();//roi is not optional in this variant
     if (roiXML.getMappingType(CiftiXML::ALONG_COLUMN) != CiftiMappingType::BRAIN_MODELS) throw AlgorithmException("cifti roi does not have brain models mapping along column");
@@ -497,13 +513,13 @@ AlgorithmCiftiCorrelation::AlgorithmCiftiCorrelation(ProgressObject* myProgObj,
         AlgorithmCiftiSeparate(NULL, ciftiRoi, CiftiXML::ALONG_COLUMN, &volRoi, offsetOut, NULL, false);//don't crop, because it needs to match the original volume space in the input
         volRoiPtr = &volRoi;
     }
-    AlgorithmCiftiCorrelation(myProgObj, myCifti, myCiftiOut, leftRoiPtr, rightRoiPtr, cerebRoiPtr, volRoiPtr, weights, fisherZ, memLimitGB);//HACK: pass through our progress object
+    AlgorithmCiftiCorrelation(myProgObj, myCifti, myCiftiOut, leftRoiPtr, rightRoiPtr, cerebRoiPtr, volRoiPtr, weights, fisherZ, memLimitGB, noDemean, covariance);//HACK: pass through our progress object
 }
 
 float AlgorithmCiftiCorrelation::correlate(const float* row1, const float& rrs1, const float* row2, const float& rrs2, const bool& fisherZ)
 {
     double r;
-    if (row1 == row2)
+    if (row1 == row2 && !m_covariance)
     {
         r = 1.0;//short circuit for same row
     } else {
@@ -515,30 +531,50 @@ float AlgorithmCiftiCorrelation::correlate(const float* row1, const float& rrs1,
             {
                 accum += row1[i] * row2[i];//these have already had the weighted row means subtracted out, and weights applied
             }
-            r = accum / (rrs1 * rrs2);//as do these
+            if (m_covariance)
+            {
+                if (m_binaryWeights)
+                {
+                    r = accum / numWeights;
+                } else {
+                    r = accum / rrs1;//NOTE: will equal rrs2 as it only depends on weights, and is not square root
+                }
+            } else {
+                r = accum / (rrs1 * rrs2);//as do these
+            }
         } else {
             double accum = 0.0;
             for (int i = 0; i < m_numCols; ++i)
             {
                 accum += row1[i] * row2[i];//these have already had the row means subtracted out
             }
-            r = accum / (rrs1 * rrs2);
+            if (m_covariance)
+            {
+                r = accum / m_numCols;
+            } else {
+                r = accum / (rrs1 * rrs2);
+            }
         }
     }
-    if (fisherZ)
+    if (!m_covariance)
     {
-        if (r > 0.999999) r = 0.999999;//prevent inf
-        if (r < -0.999999) r = -0.999999;//prevent -inf
-        return 0.5 * log((1 + r) / (1 - r));
-    } else {
-        if (r > 1.0) r = 1.0;//don't output anything silly
-        if (r < -1.0) r = -1.0;
-        return r;
+        if (fisherZ)
+        {
+            if (r > 0.999999) r = 0.999999;//prevent inf
+            if (r < -0.999999) r = -0.999999;//prevent -inf
+            r = 0.5 * log((1 + r) / (1 - r));
+        } else {
+            if (r > 1.0) r = 1.0;//don't output anything silly
+            if (r < -1.0) r = -1.0;
+        }
     }
+    return r;
 }
 
-void AlgorithmCiftiCorrelation::init(const CiftiFile* input, const vector<float>* weights)
+void AlgorithmCiftiCorrelation::init(const CiftiFile* input, const vector<float>* weights, const bool& noDemean, const bool& covariance)
 {
+    m_noDemean = noDemean;
+    m_covariance = covariance;
     m_inputCifti = input;
     m_rowInfo.resize(m_inputCifti->getNumberOfRows());
     m_cacheUsed = 0;
@@ -638,65 +674,85 @@ const float* AlgorithmCiftiCorrelation::getRow(const int& ciftiIndex, float& roo
 void AlgorithmCiftiCorrelation::computeRowStats(const float* row, float& mean, float& rootResidSqr)
 {
     double accum = 0.0;//double, for numerical stability
-    if (m_weightedMode)
+    if (m_noDemean)
     {
-        int weightsize = (int)m_weightIndexes.size();
-        if (m_binaryWeights)//because should be a little faster without multiplies or a second sum
-        {
-            for (int i = 0; i < weightsize; ++i)
-            {
-                accum += row[m_weightIndexes[i]];
-            }
-            mean = accum / weightsize;
-        } else {
-            double accum2 = 0.0;
-            for (int i = 0; i < weightsize; ++i)
-            {
-                float weight = m_weights[i];
-                accum += row[m_weightIndexes[i]] * weight;
-                accum2 += weight;
-            }
-            mean = accum / accum2;
-        }
+        mean = 0.0f;
     } else {
-        for (int i = 0; i < m_numCols; ++i)//two pass, for numerical stability
+        if (m_weightedMode)
         {
-            accum += row[i];
+            int weightsize = (int)m_weightIndexes.size();
+            if (m_binaryWeights)//because should be a little faster without multiplies or a second sum
+            {
+                for (int i = 0; i < weightsize; ++i)
+                {
+                    accum += row[m_weightIndexes[i]];
+                }
+                mean = accum / weightsize;
+            } else {
+                double accum2 = 0.0;
+                for (int i = 0; i < weightsize; ++i)
+                {
+                    float weight = m_weights[i];
+                    accum += row[m_weightIndexes[i]] * weight;
+                    accum2 += weight;
+                }
+                mean = accum / accum2;
+            }
+        } else {
+            for (int i = 0; i < m_numCols; ++i)//two pass, for numerical stability
+            {
+                accum += row[i];
+            }
+            mean = accum / m_numCols;
         }
-        mean = accum / m_numCols;
     }
     accum = 0.0;
-    if (m_weightedMode)
+    if (m_covariance)
     {
         int weightsize = (int)m_weightIndexes.size();
-        if (m_binaryWeights)
+        rootResidSqr = 0.0f;
+        if (m_weightedMode && !m_binaryWeights)
         {
             for (int i = 0; i < weightsize; ++i)
             {
-                float tempf = row[m_weightIndexes[i]] - mean;
-                accum += tempf * tempf;
+                accum += m_weights[i];
             }
-           rootResidSqr = sqrt(accum);
-        } else {
-            for (int i = 0; i < weightsize; ++i)
+            rootResidSqr = accum;//repurpose this variable to store the weight sum - NOTE: don't take sqrt in case negative sum (whatever that means), so must not divide by both in correlate() in covariance mode
+        }
+    } else {
+        if (m_weightedMode)
+        {
+            int weightsize = (int)m_weightIndexes.size();
+            if (m_binaryWeights)
             {
-                float tempf = row[m_weightIndexes[i]] - mean;
-                accum += tempf * tempf * m_weights[i];
+                for (int i = 0; i < weightsize; ++i)
+                {
+                    float tempf = row[m_weightIndexes[i]] - mean;
+                    accum += tempf * tempf;
+                }
+            rootResidSqr = sqrt(accum);
+            } else {
+                for (int i = 0; i < weightsize; ++i)
+                {
+                    float tempf = row[m_weightIndexes[i]] - mean;
+                    accum += tempf * tempf * m_weights[i];
+                }
+                rootResidSqr = sqrt(accum);
+            }
+        } else {
+            for (int i = 0; i < m_numCols; ++i)
+            {
+                float tempf = row[i] - mean;
+                accum += tempf * tempf;
             }
             rootResidSqr = sqrt(accum);
         }
-    } else {
-        for (int i = 0; i < m_numCols; ++i)
-        {
-            float tempf = row[i] - mean;
-            accum += tempf * tempf;
-        }
-        rootResidSqr = sqrt(accum);
     }
 }
 
 void AlgorithmCiftiCorrelation::doSubtract(float* row, const float& mean)
 {
+    if (m_noDemean) return;//skip subtracting zero from everything
     if (m_weightedMode)
     {
         int weightsize = (int)m_weightIndexes.size();
