@@ -41,6 +41,7 @@
 
 
 class QGLWidget;
+class QImage;
 
 namespace caret {
     
@@ -60,6 +61,7 @@ namespace caret {
     class SelectionItem;
     class SelectionManager;
     class IdentificationWithColor;
+    class ImageFile;
     class Plane;
     class Surface;
     class Model;
@@ -115,12 +117,6 @@ namespace caret {
         virtual AString getStateOfOpenGL() const;
         
     private:
-//        enum ClippingDataType {
-//            CLIPPING_DATA_TYPE_FEATURES,
-//            CLIPPING_DATA_TYPE_SURFACE,
-//            CLIPPING_DATA_TYPE_,
-//            CLIPPI
-//        }
         class VolumeDrawInfo {
         public:
             VolumeDrawInfo(CaretMappableDataFile* mapFile,
@@ -163,13 +159,6 @@ namespace caret {
                                             const StructureEnum::Enum structure,
                                             FiberTrajectoryColorModel::Item* colorSource,
                                             FiberOrientationDisplayInfo& dispInfo);
-        
-//        void colorizeVoxels(const VolumeDrawInfo& volumeDrawInfo,
-//                            const float* scalarValues,
-//                            const float* thresholdValues,
-//                            const int32_t numberOfScalars,
-//                            float* rgbaOut,
-//                            const bool ignoreThresholding);
 
         void drawModelInternal(Mode mode,
                                BrainOpenGLViewportContent* viewportContent);
@@ -299,9 +288,6 @@ namespace caret {
                                      const int32_t viewport[4]);
         
         
-//        void setOrthographicProjection(const int32_t viewport[4],
-//                                       const Model::ViewingTransformIndex rotationMatrixIndex);
-        
         void setOrthographicProjection(const int32_t viewport[4],
                                        const ProjectionViewTypeEnum::Enum projectionType);
         
@@ -350,7 +336,7 @@ namespace caret {
         void setViewportAndOrthographicProjection(const int32_t viewport[4],
                                                   const  ProjectionViewTypeEnum::Enum projectionType);
         
-        void setViewportAndOrthographicProjectionForVolume(const int32_t viewport[4],
+        void setViewportAndOrthographicProjectionForWholeBrainVolume(const int32_t viewport[4],
                                                            const  ProjectionViewTypeEnum::Enum projectionType,
                                                            const VolumeMappableInterface* volume);
         
@@ -440,9 +426,10 @@ namespace caret {
                          const FastStatistics* statistics,
                          const int paletteDrawingIndex);
         
-//        float modelSizeToPixelSize(const float modelSize);
-//        
-//        float pixelSizeToModelSize(const float pixelSize);
+        void drawBackgroundImage(BrainOpenGLViewportContent* vpContent);
+        
+        void drawImage(const int viewport[4],
+                       const ImageFile* imageFile);
         
         void setProjectionModeData(const float screenDepth,
                                           const float xyz[3],
@@ -567,15 +554,16 @@ namespace caret {
         static float COLOR_BLUE[3];
         
         friend class BrainOpenGLChartDrawingFixedPipeline;
+        friend class BrainOpenGLVolumeObliqueSliceDrawing;
         friend class BrainOpenGLVolumeSliceDrawing;
-        friend class OLD_BrainOpenGLVolumeSliceDrawing;
+        friend class OldBrainOpenGLVolumeSliceDrawing;
     };
 
 #ifdef __BRAIN_OPENGL_FIXED_PIPELINE_DEFINE_H
     bool BrainOpenGLFixedPipeline::s_staticInitialized = false;
     float BrainOpenGLFixedPipeline::COLOR_RED[3]   = { 1.0, 0.0, 0.0 };
-    float BrainOpenGLFixedPipeline::COLOR_GREEN[3]  = { 0.0, 1.0, 0.0 };
-    float BrainOpenGLFixedPipeline::COLOR_BLUE[3] = { 0.0, 0.0, 1.0 };
+    float BrainOpenGLFixedPipeline::COLOR_GREEN[3] = { 0.0, 1.0, 0.0 };
+    float BrainOpenGLFixedPipeline::COLOR_BLUE[3]  = { 0.0, 0.0, 1.0 };
 #endif //__BRAIN_OPENGL_FIXED_PIPELINE_DEFINE_H
 
 } // namespace
