@@ -22,6 +22,7 @@
 
 #include "CaretAssertion.h"
 #include "CommandClassCreate.h"
+#include "DataFileException.h"
 #include "FileInformation.h"
 #include "ProgramParameters.h"
 #include "TextFile.h"
@@ -116,8 +117,7 @@ CommandClassCreate::getHelpInformation(const AString& /*programName*/)
  *   If there is an error in the parameters.
  */
 void 
-CommandClassCreate::executeOperation(ProgramParameters& parameters) throw (CommandException,
-                                                               ProgramParametersException)
+CommandClassCreate::executeOperation(ProgramParameters& parameters)
 {
     const AString className = parameters.nextString("Class Name");
     AString derivedFromClassName = "CaretObject";
