@@ -34,9 +34,7 @@ public:
 
     virtual ~CommandGiftiConvert();
 
-    virtual void executeOperation(ProgramParameters& parameters)
-    throw (CommandException,
-           ProgramParametersException);
+    virtual void executeOperation(ProgramParameters& parameters);
 
     AString getHelpInformation(const AString& /*programName*/);
 
