@@ -31,6 +31,53 @@ std::ostream& operator << (std::ostream &lhs, const AString &rhs)
     return lhs << rhs.toStdString(); 
 }
 
+/**
+ * Convert a vector of values into a string.
+ * @param v
+ *   The vector of values.
+ * @param separator
+ *   Inserted between each pair of values.
+ * @return
+ *   String containing the vector's values separated
+ *   by the separator.
+ */
+AString
+AString::fromNumbers(const std::vector<int8_t>& v, const AString& separator)
+{
+    AString s;
+    for (uint64_t i = 0; i < v.size(); i++) {
+        if (i > 0) {
+            s += separator;
+        }
+        s += AString::number(v[i]);
+    }
+    return s;
+}
+
+/**
+ * Convert a vector of values into a string.
+ * @param v
+ *   The vector of values.
+ * @param separator
+ *   Inserted between each pair of values.
+ * @return
+ *   String containing the vector's values separated
+ *   by the separator.
+ */
+AString
+AString::fromNumbers(const std::vector<uint8_t>& v, const AString& separator)
+{
+    AString s;
+    for (uint64_t i = 0; i < v.size(); i++) {
+        if (i > 0) {
+            s += separator;
+        }
+        s += AString::number(v[i]);
+    }
+    return s;
+}
+
+
 /**
  * Convert a vector of values into a string.
  * @param v
@@ -195,6 +242,61 @@ AString::fromNumbers(const float* array, const int64_t numberOfElements, const A
     return s;
 }
 
+/**
+ * Convert an array of values into a string.
+ * @param array
+ *   The array of values.
+ * @param numberOfElements
+ *   Number of elements in the array.
+ * @param separator
+ *   Inserted between each pair of values.
+ * @return
+ *   String containing the array values separated
+ *   by the separator.
+ */
+AString
+AString::fromNumbers(const int8_t* array,
+                     const int64_t numberOfElements,
+                     const AString& separator)
+{
+    AString s;
+    for (int64_t i = 0; i < numberOfElements; i++) {
+        if (i > 0) {
+            s += separator;
+        }
+        s += AString::number(array[i]);
+    }
+    return s;
+}
+
+/**
+ * Convert an array of values into a string.
+ * @param array
+ *   The array of values.
+ * @param numberOfElements
+ *   Number of elements in the array.
+ * @param separator
+ *   Inserted between each pair of values.
+ * @return
+ *   String containing the array values separated
+ *   by the separator.
+ */
+AString
+AString::fromNumbers(const uint8_t* array,
+                     const int64_t numberOfElements,
+                     const AString& separator)
+{
+    AString s;
+    for (int64_t i = 0; i < numberOfElements; i++) {
+        if (i > 0) {
+            s += separator;
+        }
+        s += AString::number(array[i]);
+    }
+    return s;
+}
+
+
 /**
  * Convert an array of values into a string.
  * @param array
@@ -621,10 +723,15 @@ AString::replaceHtmlSpecialCharactersWithEscapeCharacters() const
  */
 int32_t
 AString::indexOfAnyChar(const AString& str,
-                        const int from) const
+                        int from) const
 {
     const AString& s = *this;
     const int len = s.length();
+    if (from < 0)//use the same "from" logic as Qt
+    {
+        from += len;//-2 starts at second to last character
+        if (from < 0) from = 0;//can't start before the beginning
+    }
     const int len2 = str.length();
     for (int i = from; i < len; i++) {
         for (int j = 0; j < len2; j++) {
@@ -726,3 +833,36 @@ AString::appendWithNewLine(const AString& str)
     append(str);
 }
 
+/**
+ * Count the number of matching characters with the other string
+ * starting at the end of the strings.
+ * 
+ * Example: this="someText"  rhs="moreText" => result=4
+ * @rhs 
+ *    The other string.
+ * @return
+ *    Number of character that match from the end of this and rhs.
+ */
+int64_t
+AString::countMatchingCharactersFromEnd(const AString& rhs) const
+{
+    int64_t matchCount = 0;
+    
+    const int64_t myLength  = length();
+    const int64_t rhsLength = rhs.length();
+    const int64_t minLength  = std::min(myLength,
+                                       rhsLength);
+    
+    for (int32_t offset = 1; offset <= minLength; offset++) {
+        if (at(myLength - offset) == rhs.at(rhsLength - offset)) {
+            matchCount++;
+        }
+        else {
+            break;
+        }
+    }
+
+    return matchCount;
+}
+
+
