@@ -250,25 +250,25 @@ main(int argc, char* argv[])
         /*
         * Handle uncaught exceptions
         */
-        SystemUtilities::setHandlersForUnexpected();
+        SystemUtilities::setUnexpectedHandler();
         
         /*
         * Create the session manager.
         */
-        SessionManager::createSessionManager();
+        SessionManager::createSessionManager(ApplicationTypeEnum::APPLICATION_TYPE_GRAPHICAL_USER_INTERFACE);
         caretLoggerIsValid = true;
 
         /*
         * Parameters for the program.
         */
-        ProgramParameters* parameters = new ProgramParameters(argc, argv);
-        caret_global_commandLine = AString(argv[0]) + " " + parameters->getAllParametersInString();
+        ProgramParameters parameters(argc, argv);
+        caret_global_commandLine_init(parameters);
 
         //begin parsing command line
         ProgramState myState;
         FileInformation progInfo(argv[0]);
         AString progName = progInfo.getFileName();
-        parseCommandLine(progName, parameters, myState);
+        parseCommandLine(progName, &parameters, myState);
         
         /*
         * Log the command parameters.
@@ -594,8 +594,6 @@ main(int argc, char* argv[])
         SessionManager::deleteSessionManager();
         
         CaretHttpManager::deleteHttpManager();
-        
-        delete parameters;
     }
     /*
      * See if any objects were not deleted.
@@ -663,7 +661,10 @@ void printHelp(const AString& progName)
     }
     
     cout
-    << "        other styles may be available on other systems" << endl
+    << "        The selected style is listed on the About wb_view dialog" << endl
+    << "        available from the File Menu (On Macs: wb_view Menu). " << endl
+    << "        Press the \"More\" button to see the selected style." << endl
+    << "        Other styles may be available on other systems." << endl
     << endl
     << "    -spec-load-all" << endl
     << "        load all files in the given spec file, don't show spec file dialog" << endl
