@@ -27,6 +27,7 @@
 #include "CaretLogger.h"
 #include "CaretPointer.h"
 #include "DataFileContentInformation.h"
+#include "DataFileException.h"
 #include "EventGetDisplayedDataFiles.h"
 #include "EventManager.h"
 #include "FileAdapter.h"
@@ -157,7 +158,7 @@ SpecFile::initializeSpecFile()
     this->metadata = new GiftiMetaData();
     
     std::vector<DataFileTypeEnum::Enum> allEnums;
-    DataFileTypeEnum::getAllEnums(allEnums, false, false);
+    DataFileTypeEnum::getAllEnums(allEnums, false);
     
     /*
      * Do surface files first since they need to be loaded before other files
@@ -443,7 +444,7 @@ SpecFile::addDataFile(const DataFileTypeEnum::Enum dataFileType,
                       const AString& filename,
                       const bool fileLoadingSelectionStatus,
                       const bool fileSavingSelectionStatus,
-                      const bool specFileMemberStatus) throw (DataFileException)
+                      const bool specFileMemberStatus)
 {
     addDataFilePrivate(dataFileType,
                        structure,
@@ -481,7 +482,7 @@ SpecFile::addDataFilePrivate(const DataFileTypeEnum::Enum dataFileType,
                              const AString& filename,
                              const bool fileLoadingSelectionStatus,
                              const bool fileSavingSelectionStatus,
-                             const bool specFileMemberStatus) throw (DataFileException)
+                             const bool specFileMemberStatus)
 {
     AString name = filename;
 
@@ -559,7 +560,8 @@ SpecFile::addDataFilePrivate(const DataFileTypeEnum::Enum dataFileType,
         }
     }
                         
-    DataFileException e("Data File Type: " 
+    DataFileException e(getFileName(),
+                        "Data File Type: "
                         + DataFileTypeEnum::toName(dataFileType)
                         + " not allowed "
                         + " for file "
@@ -652,7 +654,7 @@ SpecFile::addDataFile(const AString& dataFileTypeName,
                       const AString& filename,
                       const bool fileLoadingSelectionStatus,
                       const bool fileSavingSelectionStatus,
-                      const bool specFileMemberStatus) throw (DataFileException)
+                      const bool specFileMemberStatus)
 {
     bool validType = false;
     DataFileTypeEnum::Enum dataFileType = DataFileTypeEnum::fromName(dataFileTypeName, &validType);
@@ -865,7 +867,7 @@ SpecFile::removeAnyFileInformationIfNotInSpecAndNoCaretDataFile()
  *    If there is an error reading the file.
  */
 void 
-SpecFile::readFile(const AString& filenameIn) throw (DataFileException)
+SpecFile::readFile(const AString& filenameIn)
 {
     clear();
     
@@ -890,9 +892,7 @@ SpecFile::readFile(const AString& filenameIn) throw (DataFileException)
         int lineNum = e.getLineNumber();
         int colNum  = e.getColumnNumber();
         
-        AString msg =
-        "Parse Error while reading "
-        + filename;
+        AString msg = "Parse Error while reading:";
         
         if ((lineNum >= 0) && (colNum >= 0)) {
             msg += (" line/col ("
@@ -904,7 +904,8 @@ SpecFile::readFile(const AString& filenameIn) throw (DataFileException)
         
         msg += (": " + e.whatString());
         
-        DataFileException dfe(msg);
+        DataFileException dfe(filename,
+                              msg);
         CaretLogThrowing(dfe);
         throw dfe;
     }
@@ -924,7 +925,7 @@ SpecFile::readFile(const AString& filenameIn) throw (DataFileException)
  *    If there is an error reading the file from the string.
  */
 void 
-SpecFile::readFileFromString(const AString& string) throw (DataFileException)
+SpecFile::readFileFromString(const AString& string)
 {
     SpecFileSaxReader saxReader(this);
     std::auto_ptr<XmlSaxParser> parser(XmlSaxParser::createXmlParser());
@@ -950,7 +951,8 @@ SpecFile::readFileFromString(const AString& string) throw (DataFileException)
         
         msg += (": " + e.whatString());
         
-        DataFileException dfe(msg);
+        DataFileException dfe(getFileName(),
+                              msg);
         CaretLogThrowing(dfe);
         throw dfe;
     }
@@ -968,7 +970,7 @@ SpecFile::readFileFromString(const AString& string) throw (DataFileException)
  *    If there is an error writing the file.
  */
 void 
-SpecFile::writeFile(const AString& filename) throw (DataFileException)
+SpecFile::writeFile(const AString& filename)
 {
     checkFileWritability(filename);
     
@@ -990,7 +992,8 @@ SpecFile::writeFile(const AString& filename) throw (DataFileException)
         QTextStream* textStream = file.openQTextStreamForWritingFile(this->getFileName(),
                                                                      errorMessage);
         if (textStream == NULL) {
-            throw DataFileException(errorMessage);
+            throw DataFileException(getFileName(),
+                                    errorMessage);
         }
 
         //
@@ -1010,10 +1013,12 @@ SpecFile::writeFile(const AString& filename) throw (DataFileException)
         this->clearModified();
     }
     catch (const GiftiException& e) {
-        throw DataFileException(e);
+        throw DataFileException(getFileName(),
+                                e.whatString());
     }
     catch (const XmlException& e) {
-        throw DataFileException(e);
+        throw DataFileException(getFileName(),
+                                e.whatString());
     }
 }
 
@@ -1029,7 +1034,7 @@ SpecFile::writeFile(const AString& filename) throw (DataFileException)
 void 
 SpecFile::writeFileContentToXML(XmlWriter& xmlWriter,
                                 const WriteMetaDataType writeMetaDataStatus,
-                                const WriteFilesSelectedType writeFilesSelectedStatus) throw (DataFileException)
+                                const WriteFilesSelectedType writeFilesSelectedStatus)
 {    
     //
     // Write header info
@@ -1150,7 +1155,7 @@ SpecFile::updateFileNameAndPathForWriting(const AString& dataFileNameIn)
  */
 //AString 
 //SpecFile::writeFileToString(const WriteMetaDataType writeMetaDataStatus,
-//                            const WriteFilesSelectedType writeFilesSelectedStatus) throw (DataFileException)
+//                            const WriteFilesSelectedType writeFilesSelectedStatus)
 //{    
 //    /*
 //     * Create a TextStream that writes to a string.
