@@ -44,6 +44,8 @@
 #include "CaretPreferences.h"
 #include "CursorManager.h"
 #include "DummyFontTextRenderer.h"
+#include "EventBrainReset.h"
+#include "EventImageCapture.h"
 #include "EventModelGetAll.h"
 #include "EventManager.h"
 #include "EventBrowserWindowContentGet.h"
@@ -60,12 +62,14 @@
 #include "MouseEvent.h"
 #include "SelectionManager.h"
 #include "SelectionItemSurfaceNode.h"
+#include "SelectionItemVoxelEditing.h"
 #include "SessionManager.h"
 #include "Surface.h"
 #include "TileTabsConfiguration.h"
 #include "UserInputModeBorders.h"
 #include "UserInputModeFoci.h"
 #include "UserInputModeView.h"
+#include "UserInputModeVolumeEdit.h"
 
 
 using namespace caret;
@@ -117,15 +121,20 @@ BrainOpenGLWidget::BrainOpenGLWidget(QWidget* parent,
     this->userInputBordersModeProcessor = new UserInputModeBorders(this->borderBeingDrawn,
                                                                    windowIndex);
     this->userInputFociModeProcessor = new UserInputModeFoci(windowIndex);
+    this->userInputVolumeEditModeProcessor = new UserInputModeVolumeEdit(windowIndex);
     this->userInputViewModeProcessor = new UserInputModeView();
     this->selectedUserInputProcessor = this->userInputViewModeProcessor;
     this->selectedUserInputProcessor->initialize();
     this->mousePressX = -10000;
     this->mousePressY = -10000;
+    this->mouseNewDraggingStartedFlag = false;
+    
+    EventManager::get()->addEventListener(this, EventTypeEnum::EVENT_BRAIN_RESET);
     EventManager::get()->addEventListener(this, EventTypeEnum::EVENT_GRAPHICS_UPDATE_ALL_WINDOWS);
     EventManager::get()->addEventListener(this, EventTypeEnum::EVENT_GRAPHICS_UPDATE_ONE_WINDOW);
     EventManager::get()->addEventListener(this, EventTypeEnum::EVENT_GET_OR_SET_USER_INPUT_MODE);
     EventManager::get()->addEventListener(this, EventTypeEnum::EVENT_IMAGE_CAPTURE);
+    EventManager::get()->addEventListener(this, EventTypeEnum::EVENT_USER_INTERFACE_UPDATE);
 }
 
 /**
@@ -149,6 +158,7 @@ BrainOpenGLWidget::~BrainOpenGLWidget()
     delete this->userInputViewModeProcessor;
     delete this->userInputBordersModeProcessor;
     delete this->userInputFociModeProcessor;
+    delete this->userInputVolumeEditModeProcessor;
     this->selectedUserInputProcessor = NULL; // DO NOT DELETE since it does not own the object to which it points
     
     delete this->borderBeingDrawn;
@@ -285,10 +295,229 @@ BrainOpenGLWidget::clearDrawingViewportContents()
     this->drawingViewportContents.clear();
 }
 
+///**
+// * Paints the graphics.
+// */
+//void 
+//BrainOpenGLWidget::paintGL()
+//{
+//    /*
+//     * Set the cursor to that requested by the user input processor
+//     */
+//    CursorEnum::Enum cursor = this->selectedUserInputProcessor->getCursor();
+//    
+//    GuiManager::get()->getCursorManager()->setCursorForWidget(this,
+//                                                              cursor);
+//    
+//    this->clearDrawingViewportContents();
+//    
+//    int windowViewport[4] = {
+//        0,
+//        0,
+//        this->windowWidth[this->windowIndex],
+//        this->windowHeight[this->windowIndex]
+//    };
+//    
+//    EventBrowserWindowContentGet getModelEvent(this->windowIndex);
+//    EventManager::get()->sendEvent(getModelEvent.getPointer());
+//
+//    if (getModelEvent.isError()) {
+//        return;
+//    }
+//
+//    /*
+//     * Highlighting of border points
+//     */
+//    this->openGL->setDrawHighlightedEndPoints(false);
+//    if (this->selectedUserInputProcessor == this->userInputBordersModeProcessor) {
+//        this->openGL->setDrawHighlightedEndPoints(this->userInputBordersModeProcessor->isHighlightBorderEndPoints());
+//    }
+//    
+//    const int32_t numToDraw = getModelEvent.getNumberOfItemsToDraw();
+//    if (numToDraw == 1) {
+//        BrainOpenGLViewportContent* vc = new BrainOpenGLViewportContent(windowViewport,
+//                                                                        windowViewport,
+//                                                                        false,
+//                                                                        GuiManager::get()->getBrain(),
+//                                                                        getModelEvent.getTabContentToDraw(0));
+//        this->drawingViewportContents.push_back(vc);
+//    }
+//    else if (numToDraw > 1) {
+//        int32_t numRows = 0;
+//        int32_t numCols = 0;
+//        
+//        const int32_t windowWidth = this->windowWidth[this->windowIndex];
+//        const int32_t windowHeight = this->windowHeight[this->windowIndex];
+//        
+//        std::vector<int32_t> rowHeights;
+//        std::vector<int32_t> columnsWidths;
+//        
+//        /*
+//         * Determine if default configuration for tiles
+//         */
+//        TileTabsConfiguration* tileTabsConfiguration = getModelEvent.getTileTabsConfiguration();
+//        CaretAssert(tileTabsConfiguration);
+//        
+//        /*
+//         * NOTE: When computing widths and heights, do not round.
+//         * Rounding may cause the bottom most row or column to extend
+//         * outside the graphics region.  Shrinking the last row or 
+//         * column is not desired since it might cause the last model
+//         * to be drawn slightly smaller than the others.
+//         */
+//        if (tileTabsConfiguration->isDefaultConfiguration()) {
+//            /*
+//             * Update number of rows/columns in the default configuration
+//             * so that if a scene is saved, the correct number of rows
+//             * and columns are saved to the scene.
+//             */
+//            tileTabsConfiguration->updateDefaultConfigurationRowsAndColumns(numToDraw);
+//            numRows = tileTabsConfiguration->getNumberOfRows();
+//            numCols = tileTabsConfiguration->getNumberOfColumns();
+//            
+//            for (int32_t i = 0; i < numRows; i++) {
+//                rowHeights.push_back(windowHeight / numRows);
+//            }
+//            for (int32_t i = 0; i < numCols; i++) {
+//                columnsWidths.push_back(windowWidth / numCols);
+//            }
+//        }
+//        else {
+//            /*
+//             * Rows/columns from user configuration
+//             */
+//            numRows = tileTabsConfiguration->getNumberOfRows();
+//            numCols = tileTabsConfiguration->getNumberOfColumns();
+//            
+//            /*
+//             * Determine height of each row
+//             */
+//            float rowStretchTotal = 0.0;
+//            for (int32_t i = 0; i < numRows; i++) {
+//                rowStretchTotal += tileTabsConfiguration->getRowStretchFactor(i);
+//            }
+//            CaretAssert(rowStretchTotal > 0.0);
+//            for (int32_t i = 0; i < numRows; i++) {
+//                const int32_t h = static_cast<int32_t>((tileTabsConfiguration->getRowStretchFactor(i) / rowStretchTotal)
+//                                                              * windowHeight);
+//                
+//                rowHeights.push_back(h);
+//            }
+//            
+//            /*
+//             * Determine width of each column
+//             */
+//            float columnStretchTotal = 0.0;
+//            for (int32_t i = 0; i < numCols; i++) {
+//                columnStretchTotal += tileTabsConfiguration->getColumnStretchFactor(i);
+//            }
+//            CaretAssert(columnStretchTotal > 0.0);
+//            for (int32_t i = 0; i < numCols; i++) {
+//                const int32_t w = static_cast<int32_t>((tileTabsConfiguration->getColumnStretchFactor(i) / columnStretchTotal)
+//                                                              * windowWidth);
+//                columnsWidths.push_back(w);
+//            }
+//        }
+//        
+//        CaretAssert(numCols == static_cast<int32_t>(columnsWidths.size()));
+//        CaretAssert(numRows == static_cast<int32_t>(rowHeights.size()));
+//
+//        /*
+//         * Verify all rows fit within the window
+//         */
+//        int32_t rowHeightsSum = 0;
+//        for (int32_t i = 0; i < numRows; i++) {
+//            rowHeightsSum += rowHeights[i];
+//        }
+//        if (rowHeightsSum > windowHeight) {
+//            CaretLogSevere("PROGRAM ERROR: Tile Tabs total row heights exceed window height");
+//            rowHeights[numRows - 1] -= (rowHeightsSum - windowHeight);
+//        }
+//        
+//        /*
+//         * Adjust width of last column so that it does not extend beyond viewport
+//         */
+//        int32_t columnWidthsSum = 0;
+//        for (int32_t i = 0; i < numCols; i++) {
+//            columnWidthsSum += columnsWidths[i];
+//        }
+//        if (columnWidthsSum > windowWidth) {
+//            CaretLogSevere("PROGRAM ERROR: Tile Tabs total row heights exceed window height");
+//            columnsWidths[numCols - 1] = columnWidthsSum - windowWidth;
+//        }
+//        
+//        CaretLogFiner("Tile Tabs Row Heights: "
+//                       + AString::fromNumbers(rowHeights, ", "));
+//        CaretLogFiner("Tile Tabs Column Widths: "
+//                       + AString::fromNumbers(columnsWidths, ", "));
+//        
+//        /*
+//         * Arrange models left-to-right and top-to-bottom.
+//         */
+//        int32_t vpX = 0;
+//        int32_t vpY = this->windowHeight[this->windowIndex];
+//        
+//        int32_t iModel = 0;
+//        for (int32_t i = 0; i < numRows; i++) {
+//            const int32_t vpHeight = rowHeights[i];
+//            vpX = 0;
+//            vpY -= vpHeight;
+//            for (int32_t j = 0; j < numCols; j++) {
+//                const int32_t vpWidth = columnsWidths[j];
+//                if (iModel < numToDraw) {
+//                    const int modelViewport[4] = {
+//                        vpX,
+//                        vpY,
+//                        vpWidth,
+//                        vpHeight
+//                    };
+//                    
+//                    BrowserTabContent* tabContent = getModelEvent.getTabContentToDraw(iModel);
+//                    const bool highlightTab = (getModelEvent.getTabIndexForTileTabsHighlighting() == tabContent->getTabNumber());
+//                    BrainOpenGLViewportContent* vc =
+//                    new BrainOpenGLViewportContent(modelViewport,
+//                                                   modelViewport,
+//                                                   highlightTab,
+//                                                   GuiManager::get()->getBrain(),
+//                                                   tabContent);
+//                    this->drawingViewportContents.push_back(vc);
+//                }
+//                iModel++;
+//                vpX += vpWidth;
+//                
+//                if (iModel >= numToDraw) {
+//                    /*
+//                     * More cells than models for drawing so set loop
+//                     * indices so that loops terminate
+//                     */
+//                    j = numCols;
+//                    i = numRows;
+//                }
+//            }
+//        }
+//    }
+//    
+//    if (this->selectedUserInputProcessor == userInputBordersModeProcessor) {
+//        this->openGL->setBorderBeingDrawn(this->borderBeingDrawn);
+//    }
+//    else {
+//        this->openGL->setBorderBeingDrawn(NULL);
+//    }
+//    this->openGL->drawModels(this->drawingViewportContents);
+//    
+//    /*
+//     * Issue browser window redrawn event
+//     */
+//    BrainBrowserWindow* bbw = GuiManager::get()->getBrowserWindowByWindowIndex(this->windowIndex);
+//    if (bbw != NULL) {
+//        EventManager::get()->sendEvent(EventBrowserWindowGraphicsRedrawn(bbw).getPointer());
+//    }
+//}
+
 /**
  * Paints the graphics.
  */
-void 
+void
 BrainOpenGLWidget::paintGL()
 {
     /*
@@ -310,11 +539,11 @@ BrainOpenGLWidget::paintGL()
     
     EventBrowserWindowContentGet getModelEvent(this->windowIndex);
     EventManager::get()->sendEvent(getModelEvent.getPointer());
-
+    
     if (getModelEvent.isError()) {
         return;
     }
-
+    
     /*
      * Highlighting of border points
      */
@@ -333,8 +562,8 @@ BrainOpenGLWidget::paintGL()
         this->drawingViewportContents.push_back(vc);
     }
     else if (numToDraw > 1) {
-        int32_t numRows = 0;
-        int32_t numCols = 0;
+        //int32_t numRows = 0;
+        //int32_t numCols = 0;
         
         const int32_t windowWidth = this->windowWidth[this->windowIndex];
         const int32_t windowHeight = this->windowHeight[this->windowIndex];
@@ -345,168 +574,177 @@ BrainOpenGLWidget::paintGL()
         /*
          * Determine if default configuration for tiles
          */
-//        bool defaultTabsLayout = true;
         TileTabsConfiguration* tileTabsConfiguration = getModelEvent.getTileTabsConfiguration();
         CaretAssert(tileTabsConfiguration);
-//        if (tileTabsConfiguration != NULL) {
-//            if (tileTabsConfiguration->isDefaultConfiguration() == false) {
-//                defaultTabsLayout = false;
+        
+//        /*
+//         * NOTE: When computing widths and heights, do not round.
+//         * Rounding may cause the bottom most row or column to extend
+//         * outside the graphics region.  Shrinking the last row or
+//         * column is not desired since it might cause the last model
+//         * to be drawn slightly smaller than the others.
+//         */
+//        if (tileTabsConfiguration->isDefaultConfiguration()) {
+//            /*
+//             * Update number of rows/columns in the default configuration
+//             * so that if a scene is saved, the correct number of rows
+//             * and columns are saved to the scene.
+//             */
+//            tileTabsConfiguration->updateDefaultConfigurationRowsAndColumns(numToDraw);
+//            numRows = tileTabsConfiguration->getNumberOfRows();
+//            numCols = tileTabsConfiguration->getNumberOfColumns();
+//            
+//            for (int32_t i = 0; i < numRows; i++) {
+//                rowHeights.push_back(windowHeight / numRows);
+//            }
+//            for (int32_t i = 0; i < numCols; i++) {
+//                columnsWidths.push_back(windowWidth / numCols);
+//            }
+//        }
+//        else {
+//            /*
+//             * Rows/columns from user configuration
+//             */
+//            numRows = tileTabsConfiguration->getNumberOfRows();
+//            numCols = tileTabsConfiguration->getNumberOfColumns();
+//            
+//            /*
+//             * Determine height of each row
+//             */
+//            float rowStretchTotal = 0.0;
+//            for (int32_t i = 0; i < numRows; i++) {
+//                rowStretchTotal += tileTabsConfiguration->getRowStretchFactor(i);
+//            }
+//            CaretAssert(rowStretchTotal > 0.0);
+//            for (int32_t i = 0; i < numRows; i++) {
+//                const int32_t h = static_cast<int32_t>((tileTabsConfiguration->getRowStretchFactor(i) / rowStretchTotal)
+//                                                       * windowHeight);
+//                
+//                rowHeights.push_back(h);
+//            }
+//            
+//            /*
+//             * Determine width of each column
+//             */
+//            float columnStretchTotal = 0.0;
+//            for (int32_t i = 0; i < numCols; i++) {
+//                columnStretchTotal += tileTabsConfiguration->getColumnStretchFactor(i);
+//            }
+//            CaretAssert(columnStretchTotal > 0.0);
+//            for (int32_t i = 0; i < numCols; i++) {
+//                const int32_t w = static_cast<int32_t>((tileTabsConfiguration->getColumnStretchFactor(i) / columnStretchTotal)
+//                                                       * windowWidth);
+//                columnsWidths.push_back(w);
+//            }
+//        }
+//        
+//        CaretAssert(numCols == static_cast<int32_t>(columnsWidths.size()));
+//        CaretAssert(numRows == static_cast<int32_t>(rowHeights.size()));
+//        
+//        /*
+//         * Verify all rows fit within the window
+//         */
+//        int32_t rowHeightsSum = 0;
+//        for (int32_t i = 0; i < numRows; i++) {
+//            rowHeightsSum += rowHeights[i];
+//        }
+//        if (rowHeightsSum > windowHeight) {
+//            CaretLogSevere("PROGRAM ERROR: Tile Tabs total row heights exceed window height");
+//            rowHeights[numRows - 1] -= (rowHeightsSum - windowHeight);
+//        }
+//        
+//        /*
+//         * Adjust width of last column so that it does not extend beyond viewport
+//         */
+//        int32_t columnWidthsSum = 0;
+//        for (int32_t i = 0; i < numCols; i++) {
+//            columnWidthsSum += columnsWidths[i];
+//        }
+//        if (columnWidthsSum > windowWidth) {
+//            CaretLogSevere("PROGRAM ERROR: Tile Tabs total row heights exceed window height");
+//            columnsWidths[numCols - 1] = columnWidthsSum - windowWidth;
+//        }
+//        
+//        CaretLogFiner("Tile Tabs Row Heights: "
+//                      + AString::fromNumbers(rowHeights, ", "));
+//        CaretLogFiner("Tile Tabs Column Widths: "
+//                      + AString::fromNumbers(columnsWidths, ", "));
+        
+
+        /*
+         * Get the sizes of the tab tiles from the tile tabs configuration
+         */
+        if ( ! tileTabsConfiguration->getRowHeightsAndColumnWidthsForWindowSize(windowWidth,
+                                                                                windowHeight,
+                                                                                numToDraw,
+                                                                                rowHeights,
+                                                                                columnsWidths)) {
+            CaretLogSevere("Tile Tabs Row/Column sizing failed !!!");
+            return;
+        }
+        
+//        numRows = static_cast<int32_t>(rowHeights.size());
+//        numCols = static_cast<int32_t>(columnsWidths.size());
+//        
+//        /*
+//         * Arrange models left-to-right and top-to-bottom.
+//         */
+//        int32_t vpX = 0;
+//        int32_t vpY = this->windowHeight[this->windowIndex];
+//        
+//        int32_t iModel = 0;
+//        for (int32_t i = 0; i < numRows; i++) {
+//            const int32_t vpHeight = rowHeights[i];
+//            vpX = 0;
+//            vpY -= vpHeight;
+//            for (int32_t j = 0; j < numCols; j++) {
+//                const int32_t vpWidth = columnsWidths[j];
+//                if (iModel < numToDraw) {
+//                    const int modelViewport[4] = {
+//                        vpX,
+//                        vpY,
+//                        vpWidth,
+//                        vpHeight
+//                    };
+//                    
+//                    BrowserTabContent* tabContent = getModelEvent.getTabContentToDraw(iModel);
+//                    const bool highlightTab = (getModelEvent.getTabIndexForTileTabsHighlighting() == tabContent->getTabNumber());
+//                    BrainOpenGLViewportContent* vc =
+//                    new BrainOpenGLViewportContent(modelViewport,
+//                                                   modelViewport,
+//                                                   highlightTab,
+//                                                   GuiManager::get()->getBrain(),
+//                                                   tabContent);
+//                    this->drawingViewportContents.push_back(vc);
+//                }
+//                iModel++;
+//                vpX += vpWidth;
+//                
+//                if (iModel >= numToDraw) {
+//                    /*
+//                     * More cells than models for drawing so set loop
+//                     * indices so that loops terminate
+//                     */
+//                    j = numCols;
+//                    i = numRows;
+//                }
 //            }
 //        }
         
         /*
-         * NOTE: When computing widths and heights, do not round.
-         * Rounding may cause the bottom most row or column to extend
-         * outside the graphics region.  Shrinking the last row or 
-         * column is not desired since it might cause the last model
-         * to be drawn slightly smaller than the others.
+         * Create the viewport drawing contents for all tabs
          */
-//        if (defaultTabsLayout) {
-        if (tileTabsConfiguration->isDefaultConfiguration()) {
-//            /**
-//             * Determine the number of rows and columns for the montage.
-//             * Since screen width typically exceeds height, always have
-//             * columns greater than or equal to rows.
-//             */
-//            numRows = (int)std::sqrt((double)numToDraw);
-//            numCols = numRows;
-//            int32_t row2 = numRows * numRows;
-//            if (row2 < numToDraw) {
-//                numCols++;
-//            }
-//            if ((numRows * numCols) < numToDraw) {
-//                numRows++;
-//            }
-            
-            /*
-             * Update number of rows/columns in the default configuration
-             * so that if a scene is saved, the correct number of rows
-             * and columns are saved to the scene.
-             */
-            tileTabsConfiguration->updateDefaultConfigurationRowsAndColumns(numToDraw);
-            numRows = tileTabsConfiguration->getNumberOfRows();
-            numCols = tileTabsConfiguration->getNumberOfColumns();
-            
-            for (int32_t i = 0; i < numRows; i++) {
-                rowHeights.push_back(windowHeight / numRows);
-            }
-            for (int32_t i = 0; i < numCols; i++) {
-                columnsWidths.push_back(windowWidth / numCols);
-            }
-        }
-        else {
-            /*
-             * Rows/columns from user configuration
-             */
-            numRows = tileTabsConfiguration->getNumberOfRows();
-            numCols = tileTabsConfiguration->getNumberOfColumns();
-            
-            /*
-             * Determine height of each row
-             */
-            float rowStretchTotal = 0.0;
-            for (int32_t i = 0; i < numRows; i++) {
-                rowStretchTotal += tileTabsConfiguration->getRowStretchFactor(i);
-            }
-            CaretAssert(rowStretchTotal > 0.0);
-            for (int32_t i = 0; i < numRows; i++) {
-                const int32_t h = static_cast<int32_t>((tileTabsConfiguration->getRowStretchFactor(i) / rowStretchTotal)
-                                                              * windowHeight);
-                
-                rowHeights.push_back(h);
-            }
-            
-            /*
-             * Determine width of each column
-             */
-            float columnStretchTotal = 0.0;
-            for (int32_t i = 0; i < numCols; i++) {
-                columnStretchTotal += tileTabsConfiguration->getColumnStretchFactor(i);
-            }
-            CaretAssert(columnStretchTotal > 0.0);
-            for (int32_t i = 0; i < numCols; i++) {
-                const int32_t w = static_cast<int32_t>((tileTabsConfiguration->getColumnStretchFactor(i) / columnStretchTotal)
-                                                              * windowWidth);
-                columnsWidths.push_back(w);
-            }
-        }
-        
-        CaretAssert(numCols == static_cast<int32_t>(columnsWidths.size()));
-        CaretAssert(numRows == static_cast<int32_t>(rowHeights.size()));
-
-        /*
-         * Verify all rows fit within the window
-         */
-        int32_t rowHeightsSum = 0;
-        for (int32_t i = 0; i < numRows; i++) {
-            rowHeightsSum += rowHeights[i];
-        }
-        if (rowHeightsSum > windowHeight) {
-            CaretLogSevere("PROGRAM ERROR: Tile Tabs total row heights exceed window height");
-            rowHeights[numRows - 1] -= (rowHeightsSum - windowHeight);
-        }
-        
-        /*
-         * Adjust width of last column so that it does not extend beyond viewport
-         */
-        int32_t columnWidthsSum = 0;
-        for (int32_t i = 0; i < numCols; i++) {
-            columnWidthsSum += columnsWidths[i];
-        }
-        if (columnWidthsSum > windowWidth) {
-            CaretLogSevere("PROGRAM ERROR: Tile Tabs total row heights exceed window height");
-            columnsWidths[numCols - 1] = columnWidthsSum - windowWidth;
-        }
-        
-        CaretLogFiner("Tile Tabs Row Heights: "
-                       + AString::fromNumbers(rowHeights, ", "));
-        CaretLogFiner("Tile Tabs Column Widths: "
-                       + AString::fromNumbers(columnsWidths, ", "));
-        
-        /*
-         * Arrange models left-to-right and top-to-bottom.
-         */
-        int32_t vpX = 0;
-        int32_t vpY = this->windowHeight[this->windowIndex];
-        
-        int32_t iModel = 0;
-        for (int32_t i = 0; i < numRows; i++) {
-            const int32_t vpHeight = rowHeights[i];
-            vpX = 0;
-            vpY -= vpHeight;
-            for (int32_t j = 0; j < numCols; j++) {
-                const int32_t vpWidth = columnsWidths[j];
-                if (iModel < numToDraw) {
-                    const int modelViewport[4] = {
-                        vpX,
-                        vpY,
-                        vpWidth,
-                        vpHeight
-                    };
-                    
-                    BrowserTabContent* tabContent = getModelEvent.getTabContentToDraw(iModel);
-                    const bool highlightTab = (getModelEvent.getTabIndexForTileTabsHighlighting() == tabContent->getTabNumber());
-                    BrainOpenGLViewportContent* vc =
-                    new BrainOpenGLViewportContent(modelViewport,
-                                                   modelViewport,
-                                                   highlightTab,
-                                                   GuiManager::get()->getBrain(),
-                                                   tabContent);
-                    this->drawingViewportContents.push_back(vc);
-                }
-                iModel++;
-                vpX += vpWidth;
-                
-                if (iModel >= numToDraw) {
-                    /*
-                     * More cells than models for drawing so set loop
-                     * indices so that loops terminate
-                     */
-                    j = numCols;
-                    i = numRows;
-                }
-            }
+        std::vector<BrowserTabContent*> allTabs;
+        for (int32_t i = 0; i < getModelEvent.getNumberOfItemsToDraw(); i++) {
+            allTabs.push_back(getModelEvent.getTabContentToDraw(i));
         }
+        this->drawingViewportContents = BrainOpenGLViewportContent::createViewportContentForTileTabs(allTabs,
+                                                                                                     GuiManager::get()->getBrain(),
+                                                                                                     windowWidth,
+                                                                                                     windowHeight,
+                                                                                                     rowHeights,
+                                                                                                     columnsWidths,
+                                                                                                     getModelEvent.getTabIndexForTileTabsHighlighting());
     }
     
     if (this->selectedUserInputProcessor == userInputBordersModeProcessor) {
@@ -631,7 +869,8 @@ BrainOpenGLWidget::wheelEvent(QWheelEvent* we)
                           0,
                           delta,
                           0,
-                          0);
+                          0,
+                          this->mouseNewDraggingStartedFlag);
     this->selectedUserInputProcessor->mouseLeftDragWithCtrl(mouseEvent);
     
     we->accept();
@@ -659,7 +898,7 @@ BrainOpenGLWidget::checkForMiddleMouseButton(Qt::MouseButtons& mouseButtons,
     if (isMouseMoving) {
         if (button == Qt::NoButton) {
             if (mouseButtons == Qt::MiddleButton) {
-                if (keyModifiers == Qt::NoButton) {
+                if (keyModifiers == Qt::NoModifier) {
                     mouseButtons = Qt::LeftButton;
                     button = Qt::NoButton;
                     keyModifiers = Qt::ShiftModifier;
@@ -669,7 +908,7 @@ BrainOpenGLWidget::checkForMiddleMouseButton(Qt::MouseButtons& mouseButtons,
     }
     else {
         if (button == Qt::MiddleButton) {
-            if (keyModifiers == Qt::NoButton) {
+            if (keyModifiers == Qt::NoModifier) {
                 button = Qt::LeftButton;
                 keyModifiers = Qt::ShiftModifier;
             }
@@ -694,6 +933,12 @@ BrainOpenGLWidget::mousePressEvent(QMouseEvent* me)
                               keyModifiers,
                               false);
     
+    /*
+     * When the mouse is dragged, a mouse input receiver may want to
+     * know that a new dragging has started.
+     */
+    this->mouseNewDraggingStartedFlag = true;
+    
     this->isMousePressedNearToolBox = false;
     
     if (button == Qt::LeftButton) {
@@ -784,14 +1029,19 @@ BrainOpenGLWidget::mouseReleaseEvent(QMouseEvent* me)
                                   dx,
                                   dy,
                                   this->mousePressX,
-                                  this->mousePressY);
+                                  this->mousePressY,
+                                  this->mouseNewDraggingStartedFlag);
             
-            if (keyModifiers == Qt::NoButton) {
+            if (keyModifiers == Qt::NoModifier) {
                 this->selectedUserInputProcessor->mouseLeftClick(mouseEvent);
             }
             else if (keyModifiers == Qt::ShiftModifier) {
                 this->selectedUserInputProcessor->mouseLeftClickWithShift(mouseEvent);
             }
+            else if (keyModifiers == (Qt::ShiftModifier
+                                      | Qt::ControlModifier)) {
+                this->selectedUserInputProcessor->mouseLeftClickWithCtrlShift(mouseEvent);
+            }
         }
     }
     
@@ -830,7 +1080,7 @@ BrainOpenGLWidget::getViewportContentAtXY(const int x,
 }
 
 /**
- * Perform identification.
+ * Perform identification on all items EXCEPT voxel editing.
  *
  * @param x
  *    X-coordinate for identification.
@@ -857,6 +1107,8 @@ BrainOpenGLWidget::performIdentification(const int x,
     CaretLogFine("Performing selection");
     SelectionManager* idManager = GuiManager::get()->getBrain()->getSelectionManager();
     idManager->reset();
+    idManager->setAllSelectionsEnabled(true);
+    idManager->getVoxelEditingIdentification()->setEnabledForSelection(false);
     
     if (idViewport != NULL) {
         /*
@@ -875,7 +1127,52 @@ BrainOpenGLWidget::performIdentification(const int x,
     return idManager;
 }
 
-void 
+/**
+ * Perform identification on all items EXCEPT voxel editing.
+ *
+ * @param editingVolumeFile
+ *    Volume file that is being edited.
+ * @param x
+ *    X-coordinate for identification.
+ * @param y
+ *    Y-coordinate for identification.
+ * @return
+ *    SelectionManager providing identification information.
+ */
+SelectionManager*
+BrainOpenGLWidget::performIdentificationVoxelEditing(VolumeFile* editingVolumeFile,
+                                                     const int x,
+                                                     const int y)
+{
+    BrainOpenGLViewportContent* idViewport = this->getViewportContentAtXY(x, y);
+    
+    this->makeCurrent();
+    CaretLogFine("Performing selection");
+    SelectionManager* idManager = GuiManager::get()->getBrain()->getSelectionManager();
+    idManager->reset();
+    idManager->setAllSelectionsEnabled(false);
+    SelectionItemVoxelEditing* idVoxelEdit = idManager->getVoxelEditingIdentification();
+    idVoxelEdit->setEnabledForSelection(true);
+    idVoxelEdit->setVolumeFileForEditing(editingVolumeFile);
+    
+    if (idViewport != NULL) {
+        /*
+         * ID coordinate needs to be relative to the viewport
+         *
+         int vp[4];
+         idViewport->getViewport(vp);
+         const int idX = x - vp[0];
+         const int idY = y - vp[1];
+         */
+        this->openGL->selectModel(idViewport,
+                                  x,
+                                  y,
+                                  true);
+    }
+    return idManager;
+}
+
+void
 BrainOpenGLWidget::performProjection(const int x,
                                      const int y,
                                      SurfaceProjectedItem& projectionOut)
@@ -953,9 +1250,10 @@ BrainOpenGLWidget::mouseMoveEvent(QMouseEvent* me)
                                       dx,
                                       dy,
                                       this->mousePressX,
-                                      this->mousePressY);
+                                      this->mousePressY,
+                                      this->mouseNewDraggingStartedFlag);
 
-                if (keyModifiers == Qt::NoButton) {
+                if (keyModifiers == Qt::NoModifier) {
                     this->selectedUserInputProcessor->mouseLeftDrag(mouseEvent);
                 }
                 else if (keyModifiers == Qt::ControlModifier) {
@@ -967,6 +1265,12 @@ BrainOpenGLWidget::mouseMoveEvent(QMouseEvent* me)
                 else if (keyModifiers == Qt::AltModifier) {
                     this->selectedUserInputProcessor->mouseLeftDragWithAlt(mouseEvent);
                 }
+                else if (keyModifiers == (Qt::ShiftModifier
+                                          | Qt::ControlModifier)) {
+                    this->selectedUserInputProcessor->mouseLeftDragWithCtrlShift(mouseEvent);
+                }
+                
+                this->mouseNewDraggingStartedFlag = false;
             }
             
             this->lastMouseX = mouseX;
@@ -986,7 +1290,15 @@ BrainOpenGLWidget::mouseMoveEvent(QMouseEvent* me)
 void 
 BrainOpenGLWidget::receiveEvent(Event* event)
 {
-    if (event->getEventType() == EventTypeEnum::EVENT_GRAPHICS_UPDATE_ALL_WINDOWS) {
+    if (event->getEventType() == EventTypeEnum::EVENT_BRAIN_RESET) {
+        EventBrainReset* brainResetEvent = dynamic_cast<EventBrainReset*>(event);
+        CaretAssert(brainResetEvent);
+        
+        this->borderBeingDrawn->clear();
+        
+        brainResetEvent->setEventProcessed();
+    }
+    else if (event->getEventType() == EventTypeEnum::EVENT_GRAPHICS_UPDATE_ALL_WINDOWS) {
         EventGraphicsUpdateAllWindows* updateAllEvent =
             dynamic_cast<EventGraphicsUpdateAllWindows*>(event);
         CaretAssert(updateAllEvent);
@@ -1037,18 +1349,21 @@ BrainOpenGLWidget::receiveEvent(Event* event)
                 inputModeEvent->setUserInputProcessor(this->selectedUserInputProcessor);
             }
             else if (inputModeEvent->isSetUserInputMode()) {
-                UserInputReceiverInterface* newUserInputProcessor = NULL;
+                UserInputModeAbstract* newUserInputProcessor = NULL;
                 switch (inputModeEvent->getUserInputMode()) {
-                    case UserInputReceiverInterface::INVALID:
+                    case UserInputModeAbstract::INVALID:
                         CaretAssertMessage(0, "INVALID is NOT allowed for user input mode");
                         break;
-                    case UserInputReceiverInterface::BORDERS:
+                    case UserInputModeAbstract::BORDERS:
                         newUserInputProcessor = this->userInputBordersModeProcessor;
                         break;
-                    case UserInputReceiverInterface::FOCI:
+                    case UserInputModeAbstract::FOCI:
                         newUserInputProcessor = this->userInputFociModeProcessor;
                         break;
-                    case UserInputReceiverInterface::VIEW:
+                    case UserInputModeAbstract::VOLUME_EDIT:
+                        newUserInputProcessor = this->userInputVolumeEditModeProcessor;
+                        break;
+                    case UserInputModeAbstract::VIEW:
                         newUserInputProcessor = this->userInputViewModeProcessor;
                         break;
                 }
@@ -1073,6 +1388,13 @@ BrainOpenGLWidget::receiveEvent(Event* event)
             imageCaptureEvent->setEventProcessed();
         }
     }
+    else if (event->getEventType() == EventTypeEnum::EVENT_USER_INTERFACE_UPDATE) {
+        EventUserInterfaceUpdate* guiUpdateEvent = dynamic_cast<EventUserInterfaceUpdate*>(event);
+        CaretAssert(guiUpdateEvent);
+        guiUpdateEvent->setEventProcessed();
+        
+        this->selectedUserInputProcessor->update();
+    }
     else {
         
     }
@@ -1111,6 +1433,13 @@ BrainOpenGLWidget::captureImage(EventImageCapture* imageCaptureEvent)
     switch (imageCaptureMethod) {
         case ImageCaptureMethodEnum::IMAGE_CAPTURE_WITH_GRAB_FRAME_BUFFER:
         {
+            /*
+             * Grab frame buffer seems to have a bug in that it grabs
+             * the previous buffer on Mac so repaint to ensure frame
+             * buffer is updated.  (repaint() updates immediately,
+             * update() is a scheduled update).
+             */
+            repaint();
             image = grabFrameBuffer();
             
             /*
@@ -1145,15 +1474,15 @@ BrainOpenGLWidget::captureImage(EventImageCapture* imageCaptureEvent)
     if ((image.size().width() <= 0)
         || (image.size().height() <= 0)) {
         imageCaptureEvent->setErrorMessage("Image capture appears to have failed (invalid size).");
-        return;
     }
-    
-    imageCaptureEvent->setImage(image);
-    
-    uint8_t backgroundColor[3];
-    this->openGL->getBackgroundColor(backgroundColor);
-    imageCaptureEvent->setBackgroundColor(backgroundColor);
-    
+    else {
+        imageCaptureEvent->setImage(image);
+        
+        uint8_t backgroundColor[3];
+        this->openGL->getBackgroundColor(backgroundColor);
+        imageCaptureEvent->setBackgroundColor(backgroundColor);
+    }
+
     BrainOpenGLShape::setImmediateModeOverride(false);
     
     this->resizeGL(oldSizeX, oldSizeY);
