@@ -434,6 +434,7 @@ FociPropertiesEditorDialog::newFociFileButtonClicked()
      * Let user choose a different path/name
      */
     FociFile* newFociFile = new FociFile();
+    GuiManager::get()->getBrain()->convertDataFilePathNameToAbsolutePathName(newFociFile);
     AString newFociFileName = CaretFileDialog::getSaveFileNameDialog(DataFileTypeEnum::FOCI,
                                                                       this,
                                                                       "Choose Foci File Name",
@@ -571,7 +572,7 @@ FociPropertiesEditorDialog::okButtonClicked()
     if (m_projectCheckBox->isChecked()) {
         Brain* brain = GuiManager::get()->getBrain();
         
-        std::vector<const SurfaceFile*> surfaceFiles = brain->getVolumeInteractionSurfaceFiles();
+        std::vector<const SurfaceFile*> surfaceFiles = brain->getPrimaryAnatomicalSurfaceFiles();
         
         try {
             SurfaceProjector projector(surfaceFiles);
