@@ -29,10 +29,13 @@
 #include "GiftiFile.h"
 #include "VolumeFile.h"
 
+#include <fstream>
+#include <string>
 #include <limits>
 #include <vector>
 
 #include <QFile>
+#include <QStringList>
 
 using namespace caret;
 using namespace std;
@@ -61,7 +64,9 @@ OperationParameters* OperationCiftiConvert::getParameters()
     OptionalParameter* fgresetTimeOpt = fromGiftiExt->createOptionalParameter(3, "-reset-timepoints", "reset the mapping along rows to timepoints, taking length from the gifti file");
     fgresetTimeOpt->addDoubleParameter(1, "timestep", "the desired time between frames");
     fgresetTimeOpt->addDoubleParameter(2, "timestart", "the desired time offset of the initial frame");
-    fromGiftiExt->createOptionalParameter(4, "-reset-scalars", "reset mapping along rows to scalars");
+    OptionalParameter* fgresetTimeunitsOpt = fgresetTimeOpt->createOptionalParameter(3, "-unit", "use a unit other than time");
+    fgresetTimeunitsOpt->addStringParameter(1, "unit", "unit identifier (default SECOND)");
+    fromGiftiExt->createOptionalParameter(4, "-reset-scalars", "reset mapping along rows to scalars, taking length from the gifti file");
     OptionalParameter* fromGiftiReplace = fromGiftiExt->createOptionalParameter(5, "-replace-binary", "replace data with a binary file");
     fromGiftiReplace->addStringParameter(1, "binary-in", "the binary file that contains replacement data");
     fromGiftiReplace->createOptionalParameter(2, "-flip-endian", "byteswap the binary file");
@@ -78,16 +83,42 @@ OperationParameters* OperationCiftiConvert::getParameters()
     OptionalParameter* fnresetTimeOpt = fromNifti->createOptionalParameter(4, "-reset-timepoints", "reset the mapping along rows to timepoints, taking length from the nifti file");
     fnresetTimeOpt->addDoubleParameter(1, "timestep", "the desired time between frames");
     fnresetTimeOpt->addDoubleParameter(2, "timestart", "the desired time offset of the initial frame");
-    fromNifti->createOptionalParameter(5, "-reset-scalars", "reset mapping along rows to scalars");
+    OptionalParameter* fnresetTimeunitsOpt = fnresetTimeOpt->createOptionalParameter(3, "-unit", "use a unit other than time");
+    fnresetTimeunitsOpt->addStringParameter(1, "unit", "unit identifier (default SECOND)");
+    fromNifti->createOptionalParameter(5, "-reset-scalars", "reset mapping along rows to scalars, taking length from the nifti file");
     
-    ret->setHelpText(
-        AString("This command is used to convert a full CIFTI matrix to/from formats that can be used by programs that don't understand CIFTI.  ") +
+    OptionalParameter* toText = ret->createOptionalParameter(5, "-to-text", "convert to a plain text file");
+    toText->addCiftiParameter(1, "cifti-in", "the input cifti file");
+    toText->addStringParameter(2, "text-out", "output - the output text file");
+    OptionalParameter* toTextColDelimOpt = toText->createOptionalParameter(3, "-col-delim", "choose string to put between elements in a row");
+    toTextColDelimOpt->addStringParameter(1, "delim-string", "the string to use (default is a tab character)");
+    
+    OptionalParameter* fromText = ret->createOptionalParameter(6, "-from-text", "convert from plain text to cifti");
+    fromText->addStringParameter(1, "text-in", "the input text file");
+    fromText->addCiftiParameter(2, "cifti-template", "a cifti file with the dimension(s) and mapping(s) that should be used");
+    fromText->addCiftiOutputParameter(3, "cifti-out", "the output cifti file");
+    OptionalParameter* fromTextColDelimOpt = fromText->createOptionalParameter(4, "-col-delim", "specify string that is between elements in a row");
+    fromTextColDelimOpt->addStringParameter(1, "delim-string", "the string to use (default is any whitespace)");
+    OptionalParameter* ftresetTimeOpt = fromText->createOptionalParameter(5, "-reset-timepoints", "reset the mapping along rows to timepoints, taking length from the text file");
+    ftresetTimeOpt->addDoubleParameter(1, "timestep", "the desired time between frames");
+    ftresetTimeOpt->addDoubleParameter(2, "timestart", "the desired time offset of the initial frame");
+    OptionalParameter* ftresetTimeunitsOpt = ftresetTimeOpt->createOptionalParameter(3, "-unit", "use a unit other than time");
+    ftresetTimeunitsOpt->addStringParameter(1, "unit", "unit identifier (default SECOND)");
+    fromText->createOptionalParameter(6, "-reset-scalars", "reset mapping along rows to scalars, taking length from the text file");
+    
+    AString myText = AString("This command is used to convert a full CIFTI matrix to/from formats that can be used by programs that don't understand CIFTI.  ") +
         "If you want to write an existing CIFTI file with a different CIFTI version, see -file-convert, and its -cifti-version-convert option.  " +
         "If you want part of the CIFTI file as a metric, label, or volume file, see -cifti-separate.  " +
         "If you want to create a CIFTI file from metric and/or volume files, see the -cifti-create-* commands.  " +
-        "You must specify exactly one of -to-gifti-ext, -from-gifti-ext, -to-nifti, or -from-nifti.  " +
-        "The -transpose option to -from-gifti-ext is needed if the binary file is in column-major order."
-    );
+        "You must specify exactly one of -to-gifti-ext, -from-gifti-ext, -to-nifti, -from-nifti, -to-text, or -from-text.  " +
+        "The -transpose option to -from-gifti-ext is needed if the replacement binary file is in column-major order.  " +
+        "The -unit options accept these values:\n";
+    vector<CiftiSeriesMap::Unit> units = CiftiSeriesMap::getAllUnits();
+    for (int i = 0; i < (int)units.size(); ++i)
+    {
+        myText += "\n" + CiftiSeriesMap::unitToString(units[i]);
+    }
+    ret->setHelpText(myText);
     return ret;
 }
 
@@ -99,10 +130,14 @@ void OperationCiftiConvert::useParameters(OperationParameters* myParams, Progres
     OptionalParameter* fromGiftiExt = myParams->getOptionalParameter(2);
     OptionalParameter* toNifti = myParams->getOptionalParameter(3);
     OptionalParameter* fromNifti = myParams->getOptionalParameter(4);
+    OptionalParameter* toText = myParams->getOptionalParameter(5);
+    OptionalParameter* fromText = myParams->getOptionalParameter(6);
     if (toGiftiExt->m_present) ++modes;
     if (fromGiftiExt->m_present) ++modes;
     if (toNifti->m_present) ++modes;
     if (fromNifti->m_present) ++modes;
+    if (toText->m_present) ++modes;
+    if (fromText->m_present) ++modes;
     if (modes != 1)
     {
         throw OperationException("you must specify exactly one conversion mode");
@@ -153,7 +188,15 @@ void OperationCiftiConvert::useParameters(OperationParameters* myParams, Progres
         OptionalParameter* fgresetTimeOpt = fromGiftiExt->getOptionalParameter(3);
         if (fgresetTimeOpt->m_present)
         {
-            myXML.setMap(CiftiXML::ALONG_ROW, CiftiSeriesMap(numCols, fgresetTimeOpt->getDouble(2), fgresetTimeOpt->getDouble(1)));
+            CiftiSeriesMap::Unit myUnit = CiftiSeriesMap::SECOND;
+            OptionalParameter* fgresetTimeunitsOpt = fgresetTimeOpt->getOptionalParameter(3);
+            if (fgresetTimeunitsOpt->m_present)
+            {
+                bool ok = false;
+                myUnit = CiftiSeriesMap::stringToUnit(fgresetTimeunitsOpt->getString(1), ok);
+                if (!ok) throw OperationException("unrecognized unit name: '" + fgresetTimeunitsOpt->getString(1) + "'");
+            }
+            myXML.setMap(CiftiXML::ALONG_ROW, CiftiSeriesMap(numCols, fgresetTimeOpt->getDouble(2), fgresetTimeOpt->getDouble(1), myUnit));
         } else {
             if (myXML.getMappingType(CiftiXML::ALONG_ROW) == CiftiMappingType::SERIES)
             {
@@ -171,7 +214,12 @@ void OperationCiftiConvert::useParameters(OperationParameters* myParams, Progres
             newMap.setLength(numCols);
             myXML.setMap(CiftiXML::ALONG_ROW, newMap);
         }
-        if (myXML.getDimensionLength(CiftiXML::ALONG_ROW) != numCols || myXML.getDimensionLength(CiftiXML::ALONG_COLUMN) != numRows) throw OperationException("dimensions of input gifti array do not match dimensions in the embedded Cifti XML");
+        if (myXML.getDimensionLength(CiftiXML::ALONG_ROW) != numCols || myXML.getDimensionLength(CiftiXML::ALONG_COLUMN) != numRows)
+        {
+            throw OperationException("dimensions of input gifti array (" + AString::number(numRows) + " rows, " + AString::number(numCols) + " columns)" +
+                                     " do not match dimensions of the Cifti XML (" + AString::number(myXML.getDimensionLength(CiftiXML::ALONG_COLUMN)) + " rows, " +
+                                         AString::number(myXML.getDimensionLength(CiftiXML::ALONG_ROW)) + " columns)");
+        }
         myOutFile->setCiftiXML(myXML);
         OptionalParameter* fromGiftiReplace = fromGiftiExt->getOptionalParameter(5);
         if (fromGiftiReplace->m_present)
@@ -294,11 +342,15 @@ void OperationCiftiConvert::useParameters(OperationParameters* myParams, Progres
         OptionalParameter* fnresetTimeOpt = fromNifti->getOptionalParameter(4);
         if (fnresetTimeOpt->m_present)
         {
-            CiftiSeriesMap newMap;
-            newMap.setLength(myDims[3]);
-            newMap.setStep(fnresetTimeOpt->getDouble(1));
-            newMap.setStart(fnresetTimeOpt->getDouble(2));
-            outXML.setMap(CiftiXML::ALONG_ROW, newMap);
+            CiftiSeriesMap::Unit myUnit = CiftiSeriesMap::SECOND;
+            OptionalParameter* fnresetTimeunitsOpt = fnresetTimeOpt->getOptionalParameter(3);
+            if (fnresetTimeunitsOpt->m_present)
+            {
+                bool ok = false;
+                myUnit = CiftiSeriesMap::stringToUnit(fnresetTimeunitsOpt->getString(1), ok);
+                if (!ok) throw OperationException("unrecognized unit name: '" + fnresetTimeunitsOpt->getString(1) + "'");
+            }
+            outXML.setMap(CiftiXML::ALONG_ROW, CiftiSeriesMap(myDims[3], fnresetTimeOpt->getDouble(2), fnresetTimeOpt->getDouble(1), myUnit));
         }
         if (fromNifti->getOptionalParameter(5)->m_present)
         {
@@ -308,8 +360,16 @@ void OperationCiftiConvert::useParameters(OperationParameters* myParams, Progres
             outXML.setMap(CiftiXML::ALONG_ROW, newMap);
         }
         int64_t numRows = outXML.getDimensionLength(CiftiXML::ALONG_COLUMN), numCols = outXML.getDimensionLength(CiftiXML::ALONG_ROW);
-        if (myDims[3] != numCols) throw OperationException("input nifti has the wrong size for row length");
-        if (numRows > myDims[0] * myDims[1] * myDims[2]) throw OperationException("input nifti is too small for number of rows");
+        if (myDims[3] != numCols)
+        {
+            throw OperationException("input nifti has the different size than cifti template for row length (nifti says " + AString::number(myDims[3]) +
+                                     ", cifti XML says " + AString::number(numCols) + ")");
+        }
+        if (numRows > myDims[0] * myDims[1] * myDims[2])
+        {
+            throw OperationException("input nifti is too small for column length (need at least " + AString::number(numRows) +
+                                     ", product of first three nifti dimensions is " + AString::number(myDims[0] * myDims[1] * myDims[2]) + ")");
+        }
         myCiftiOut->setCiftiXML(outXML);
         vector<float> rowscratch(numCols);
         for (int64_t i = 0; i < numRows; ++i)
@@ -321,4 +381,111 @@ void OperationCiftiConvert::useParameters(OperationParameters* myParams, Progres
             myCiftiOut->setRow(rowscratch.data(), i);
         }
     }
+    if (toText->m_present)
+    {
+        CiftiFile* ciftiIn = toText->getCifti(1);
+        AString textOutName = toText->getString(2);
+        AString delim = "\t";
+        OptionalParameter* colDelimOpt = toText->getOptionalParameter(3);
+        if (colDelimOpt->m_present)
+        {
+            delim = colDelimOpt->getString(1);
+            if (delim.isEmpty()) throw OperationException("delimiter string must not be empty");//catch some possible stupid mistakes
+        }
+        const CiftiXML& myXML = ciftiIn->getCiftiXML();
+        if (myXML.getNumberOfDimensions() != 2) throw OperationException("conversion only supported for 2D cifti");
+        if (myXML.getDimensionLength(0) < 1) throw OperationException("input cifti has zero-length rows");
+        vector<int64_t> dims = myXML.getDimensions();
+        vector<float> scratchRow(dims[0]);
+        fstream textOut(textOutName.toLocal8Bit().constData(), fstream::out | fstream::trunc | fstream::binary);//write the same file, no newline translation, regardless of OS
+        for (int64_t i = 0; i < dims[1]; ++i)
+        {
+            ciftiIn->getRow(scratchRow.data(), i);
+            textOut << AString::number(scratchRow[0]);
+            for (int64_t j = 1; j < dims[0]; ++j)
+            {
+                textOut << delim << AString::number(scratchRow[j]);
+            }
+            textOut << "\n";
+        }
+    }
+    if (fromText->m_present)
+    {
+        AString textInName = fromText->getString(1);
+        CiftiFile* ciftiTemplate = fromText->getCifti(2);
+        CiftiFile* ciftiOut = fromText->getOutputCifti(3);
+        AString delim = "";//empty is special value for "any whitespace"
+        OptionalParameter* colDelimOpt = fromText->getOptionalParameter(4);
+        if (colDelimOpt->m_present)
+        {
+            delim = colDelimOpt->getString(1);
+            if (delim.isEmpty()) throw OperationException("delimiter string must not be empty");
+        }
+        CiftiXML outXML = ciftiTemplate->getCiftiXML();
+        if (outXML.getNumberOfDimensions() != 2) throw OperationException("conversion only supported for 2D cifti");
+        int64_t numRows = outXML.getDimensionLength(CiftiXML::ALONG_COLUMN);
+        fstream textIn(textInName.toLocal8Bit().constData(), fstream::in);//do newline translation on input
+        string templine;//need to extract a row from the file first to set the row length
+        if (!getline(textIn, templine)) throw OperationException("failed to read from input text file");
+        QStringList entries;
+        if (delim.isEmpty())
+        {
+            entries = QString(templine.c_str()).split(QRegExp("\\s+"), QString::SkipEmptyParts);
+        } else {
+            entries = QString(templine.c_str()).split(delim, QString::SkipEmptyParts);
+        }
+        if (numRows < 1) throw OperationException("template cifti file has no data");//this probably throws an exception in CiftiFile, but double check
+        int textRowLength = entries.size();
+        OptionalParameter* ftresetTimeOpt = fromText->getOptionalParameter(5);
+        if (ftresetTimeOpt->m_present)
+        {
+            CiftiSeriesMap::Unit myUnit = CiftiSeriesMap::SECOND;
+            OptionalParameter* ftresetTimeunitsOpt = ftresetTimeOpt->getOptionalParameter(3);
+            if (ftresetTimeunitsOpt->m_present)
+            {
+                bool ok = false;
+                myUnit = CiftiSeriesMap::stringToUnit(ftresetTimeunitsOpt->getString(1), ok);
+                if (!ok) throw OperationException("unrecognized unit name: '" + ftresetTimeunitsOpt->getString(1) + "'");
+            }
+            outXML.setMap(CiftiXML::ALONG_ROW, CiftiSeriesMap(textRowLength, ftresetTimeOpt->getDouble(2), ftresetTimeOpt->getDouble(1), myUnit));
+        }
+        if (fromText->getOptionalParameter(6)->m_present)
+        {
+            if (ftresetTimeOpt->m_present) throw OperationException("only one of -reset-timepoints and -reset-scalars may be specified");
+            CiftiScalarsMap newMap;
+            newMap.setLength(textRowLength);
+            outXML.setMap(CiftiXML::ALONG_ROW, newMap);
+        }
+        if (textRowLength != outXML.getDimensionLength(CiftiXML::ALONG_ROW))
+        {
+            throw OperationException("text file has different row length than cifti template (text file says " + AString::number(textRowLength) +
+                                     ", cifti XML says " + AString::number(outXML.getDimensionLength(CiftiXML::ALONG_ROW)) + ")");
+        }
+        ciftiOut->setCiftiXML(outXML);
+        vector<float> temprow(textRowLength);
+        bool ok = false;
+        for (int i = 0; i < textRowLength; ++i)
+        {
+            temprow[i] = entries[i].toFloat(&ok);
+            if (!ok) throw OperationException("failed to convert text to number: '" + entries[i] + "'");
+        }
+        ciftiOut->setRow(temprow.data(), 0);
+        for (int64_t j = 1; j < numRows; ++j)
+        {
+            if (!getline(textIn, templine)) throw OperationException("failed to read from input text file (not enough rows)");
+            if (delim.isEmpty())
+            {
+                entries = QString(templine.c_str()).split(QRegExp("\\s+"), QString::SkipEmptyParts);
+            } else {
+                entries = QString(templine.c_str()).split(delim, QString::SkipEmptyParts);
+            }
+            if (entries.size() != textRowLength) throw OperationException("text file has inconsistent line length");
+            for (int i = 0; i < textRowLength; ++i)
+            {
+                temprow[i] = entries[i].toFloat(&ok);
+                if (!ok) throw OperationException("failed to convert text to number: '" + entries[i] + "'");
+            }
+            ciftiOut->setRow(temprow.data(), j);
+        }
+    }
 }
